-- Font scaling tests.
local FONT_BASE_SIZE  = 14 -- If nothing is scaled, this is the size we would like texts to be.
local SMALL_FONT_SIZE = 14
local LARGE_FONT_SIZE = 40

local DYNAMIC_FONT_EXTRA_SCALE = 2

local WORLD_SCALE_MIN = .5
local WORLD_SCALE_MAX = 2.5

local FONT_HINTING = "none" -- "none" looks better than "normal" when the font size keeps changing.

local ZOOM_CYCLE_TIME = 4.00
local TAU             = 2*math.pi

local worldScale = 1

function love.load(args)
	LG = love.graphics

	fontStaticSmall  = LG.newFont(SMALL_FONT_SIZE, FONT_HINTING)
	fontStaticLarge  = LG.newFont(LARGE_FONT_SIZE, FONT_HINTING)
	
	-- Dummies until love.update() is called:
	fontDynamic          = LG.newFont(10)
	fontDynamicWithExtra = LG.newFont(10)
	
	fontDynamicDPI          = LG.newFont(10)
	fontDynamicWithExtraDPI = LG.newFont(10)
end

function love.update(dt)
	local time    = love.timer.getTime()
	local sin     = math.sin(time * TAU / ZOOM_CYCLE_TIME)
	local value01 = (sin + 1) / 2

	worldScale = WORLD_SCALE_MIN + (WORLD_SCALE_MAX - WORLD_SCALE_MIN) * value01

	fontDynamic:release()
	fontDynamicWithExtra:release()

	fontDynamic          = LG.newFont(FONT_BASE_SIZE                            * worldScale, FONT_HINTING)
	fontDynamicWithExtra = LG.newFont(FONT_BASE_SIZE * DYNAMIC_FONT_EXTRA_SCALE * worldScale, FONT_HINTING)
	
	fontDynamicDPI          = LG.newFont(FONT_BASE_SIZE, FONT_HINTING, worldScale)
	fontDynamicWithExtraDPI = LG.newFont(FONT_BASE_SIZE, FONT_HINTING, worldScale * DYNAMIC_FONT_EXTRA_SCALE)
end

function love.draw()
	LG.translate(10, 10)
	drawTexts(worldScale, true)

	LG.translate(0, 40)
	drawTexts(WORLD_SCALE_MIN, false)

	LG.translate(0, 10)
	drawTexts(WORLD_SCALE_MAX, false)
end

function drawTexts(_worldScale, drawDynamicFonts)
	-- The small font looks good when downscaled but bad upscaled.
	local font, text = fontStaticSmall, "1. Hello, world! Small font."
	LG.setFont(font)
	LG.scale(_worldScale)
	LG.print(text, 0, 0, 0, FONT_BASE_SIZE/font:getHeight())
	LG.scale(1/_worldScale)
	LG.translate(0, 40)

	-- The large font looks good upscaled but bad downscaled.
	local font, text = fontStaticLarge, "2. Hello, world! Large font."
	LG.setFont(font)
	LG.scale(_worldScale)
	LG.print(text, 0, 0, 0, FONT_BASE_SIZE/font:getHeight())
	LG.scale(1/_worldScale)
	LG.translate(0, 40)

	if not drawDynamicFonts then  return  end

	-- The dynamic font that tries to match the current world scale the "best"
	-- is a little bit blurry and jitters a lot.
	local font, text = fontDynamic, "3. Hello, world! Dynamic font."
	LG.setFont(font)
	LG.scale(_worldScale)
	LG.print(text, 0, 0, 0, FONT_BASE_SIZE/font:getHeight())
	LG.scale(1/_worldScale)
	LG.translate(0, 40)

	-- The dynamic font with a bit larger size than the "best" size only
	-- jitters slightly while looking sharp and nice at all world scales.
	local font, text = fontDynamicWithExtra, "4. Hello, world! Dynamic font with larger size."
	LG.setFont(font)
	LG.scale(_worldScale)
	LG.print(text, 0, 0, 0, FONT_BASE_SIZE/font:getHeight())
	LG.scale(1/_worldScale)
	LG.translate(0, 40)
	
	local font, text = fontDynamicDPI, "5. Hello, world! Dynamic font using DPI."
	LG.setFont(font)
	LG.scale(_worldScale)
	LG.print(text, 0, 0)
	LG.scale(1/_worldScale)
	LG.translate(0, 40)

	local font, text = fontDynamicWithExtraDPI, "6. Hello, world! Dynamic font with larger size using DPI."
	LG.setFont(font)
	LG.scale(_worldScale)
	LG.print(text, 0, 0)
	LG.scale(1/_worldScale)
	LG.translate(0, 40)
end

function love.keypressed(key)
	if key == "escape" then
		love.event.quit()
	end
end
